#! /usr/bin/perl -w
#
# Set of common functions for certificate lifetime handling used in Nagios plugins
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#
# Changes and Modifications
# =========================
# 29-Mar-2007 - Created; 
#               Parts of Nagios utils.pm are taken in order to make 
#               plugins Nagios-agnostic
# 

package certutils;

use Date::Parse; 
use Date::Format;

use sgutils qw($GLOBUS_LOCATION %ERRORS &checkCommand);

require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(&checkCertLifetimeSSL &checkCertLifetimeFile &checkCertDiff);

my $OPENSSL = "$GLOBUS_LOCATION/bin/openssl";

#########################
##
##   Certificate functions
##
#########################

sub checkCertDiff {
    my $diff = shift;
    my $certDate = shift;
    my $warn = shift;
    my $crit = shift;
    
    if (defined $warn) {
        $warn *= 60;
    } else {
        $warn = 1209600;
    }
    
    if (defined $crit) {
        $crit *= 60;
    } else {
        $crit = 0;
    }
    
    my $state = $ERRORS{OK};
    my $answer = "Certificate";
    my $diffString;
    
    if ($diff < $crit) {
        $state = $ERRORS{CRITICAL};
    } elsif ($diff < $warn) {
        $state = $ERRORS{WARNING};
    }
    
    if ($diff < 0) {
        $answer = "Certificate has expired <diff> ago";
        $diff = -$diff;
    } else {
        $answer = "Certificate will expire in <diff>";
    }
    
    if ($diff < 60) {
        $diffString = "$diff seconds";
        chop($diffString) if ($diff == 1) ;
    } elsif ($diff < 3600) {
        $diffString = sprintf("%.2f", $diff/60) . " minutes";
    } elsif ($diff < 86400) {
        $diffString = sprintf("%.2f", $diff/3600) . " hours";
    } else {
        $diffString = sprintf("%.2f", $diff/86400) . " days";
    }
    $answer =~ s/<diff>/$diffString/;

    $answer .= " (" . time2str("%b %d %H:%M:%S %Y %Z",$certDate,"GMT") . ").\n";

    return ($state,$answer);
}

sub checkCertLifetime {
    my $cert = shift;
    my $myDate = shift || time();
    
    my $state = $ERRORS{OK};
    my $answer;
    my $res;
    
    my $certDate;
    
    checkCommand($OPENSSL);
    
    $res = `/bin/echo "$cert" | $OPENSSL x509 -enddate -noout 2>&1`;
    
    if ($?) {
        $answer = "OPENSSL ERROR: parsing certificate failed.\n";
        $state = $ERRORS{CRITICAL};
    } else {
        $res =~ s/\n//g;
        if ($res=~/^notAfter=(.*)$/) {
            # Apr 14 15:37:58 2007 GMT
            #if ( $1 =~ /^(\S+)\s+(\d+)\s+(\d+)\:(\d+)\:(\d+)\s+(\d+)\s+(\S+)$/ ) {
            #}
            $certDate = str2time($1);
            $answer = $certDate - $myDate;
            $res = $certDate;
        } else {
            $answer = "OPENSSL ERROR: got result with invalid format.\n";
            $state = $ERRORS{CRITICAL};
        }
    }

    return ($state,$answer, $res);
}

sub checkCertLifetimeSSL 
{
    my $host = shift;
    my $port = shift;
    
    my $warn = shift;
    my $crit = shift;
    
    my $pattern = shift;
    
    my $state = $ERRORS{OK};
    my $answer;
    my $res;
    
    my $cert;
    
    checkCommand($OPENSSL);
    
    open (STD, "$OPENSSL s_client -connect $host:$port -ssl3 -showcerts < /dev/null 2>/dev/null |"); 
    
    if ($?) {
        $answer = "SSL ERROR: getting certificate failed.\n";
        $state = $ERRORS{OK};
    } else {
        my $line;
        my $found = 0;
        while ($line = <STD>) {
            if ( $line =~ /^Certificate chain/) {
                $found = 1;
                last;
            }
        }
        if ($found) {
            $found = 1;
            if ($pattern) {
                $found = 0;
                while ($line = <STD>) {
                    if ($line =~ /^\s+\d+\s+s:\/.*$pattern\s/) {
                        $found = 1;
                        last;
                    }
                }
            }
            if ($found) {
                $found = 0;
                while ($line = <STD>) {
                    if ($line =~ /^-----BEGIN CERTIFICATE-----/) {
                        $found = 1;
                    }
                    if ($found) {
                        $cert .= $line;
                        last if ($line =~ /^-----END CERTIFICATE-----/);
                    }
                }
            }
        } 
        if ($found) {
            ($state, $answer, $res) = checkCertLifetime $cert;
            if ($state == $ERRORS{OK}) {
                ($state, $answer) = checkCertDiff($answer,$res,$warn,$crit);
                $res = undef;
            }
        } else {
            $answer = "SSL ERROR: wrong response from SSL server.\n";
            $state = $ERRORS{CRITICAL};
        }
    }

    close (STD);

    return ($state, $answer, $res);
}

sub checkCertLifetimeFile 
{
    my $cert = shift;
    
    my $warn = shift;
    my $crit = shift;
    
    my $state = $ERRORS{OK};
    my $answer;
    my $res;
    
    checkCommand($OPENSSL);
    
    unless (-f $cert) {
        return ($ERRORS{CRITICAL}, "Certificate $cert doesn't exist!\n");
    }
    
    unless (-r $cert) {
        return ($ERRORS{CRITICAL}, "Cannot access certificate $cert!\n");
    }
    
    unless (open (STD, $cert)) {
        return ($ERRORS{CRITICAL}, "Error openening certificate $cert!\n");
    }
    $cert = join("",<STD>);    
    close (STD);
        
    ($state, $answer, $res) = checkCertLifetime $cert;
    if ($state == $ERRORS{OK}) {
        ($state, $answer) = checkCertDiff($answer,$res,$warn,$crit);
        $res = undef;
    }
    
    return ($state, $answer, $res);
}


1;
